/*
 * [AmericanThanksgiving.java]
 *
 * Summary: Calculate when American Thanksgiving occurs.
 *
 * Copyright: (c) 1999-2011 Roedy Green, Canadian Mind Products, http://mindprod.com
 *
 * Licence: This software may be copied and used freely for any purpose but military.
 *          http://mindprod.com/contact/nonmil.html
 *
 * Requires: JDK 1.5+
 *
 * Created with: JetBrains IntelliJ IDEA IDE http://www.jetbrains.com/idea/
 *
 * Version History:
 *  1.0 1999-09-09
 */
package com.mindprod.holidays;

import com.mindprod.common11.BigDate;

/**
 * Calculate when American Thanksgiving occurs.
 *
 * @author Roedy Green, Canadian Mind Products
 * @version 1.0 1999-09-09
 * @since 1999-09-09
 */
public final class AmericanThanksgiving extends HolInfo
    {
    // -------------------------- PUBLIC INSTANCE  METHODS --------------------------


    /**
     * @inheritDoc
     */
    public String getAuthority()
        {
        return "http://www.serve.com/shea/germusa/thanksgv.htm\n"
               + "and http://members.aol.com/calebj/thanksgiving.html";
        }

    /**
     * @inheritDoc
     */
    public int getFirstYear( int base )
        {
        return 1621;
        }

    /**
     * @inheritDoc
     */
    public String getName()
        {
        return "American Thanksgiving";
        }

    /**
     * @inheritDoc
     */
    public String getRule()
        {
        return "1621         -> first Thanksgiving, precise date unknown.\n"
               + "1622         -> was no Thanksgiving.\n"
               + "1623 .. 1675 -> precise date unknown.\n"
               + "1676 .. 1862 -> June 29.\n"
               + "1863 .. 1938 -> last Thursday of November.\n"
               + "1939 .. 1941 -> 2nd to last Thursday of November.\n"
               + "1942 .. now  -> 4th Thursday of November.";
        }

    /**
     * @inheritDoc
     */
    public int when( int year, boolean shift, int base )
        {
        if ( !isYearValid( year, base ) )
            {
            return BigDate.NULL_ORDINAL;
            }
        if ( year < 1621 || year == 1622 )
            {
            return BigDate.NULL_ORDINAL;
            }
        if ( 1621 <= year && year <= 1675 )
            {
            return BigDate.NULL_ORDINAL;
            }
        if ( 1676 <= year && year <= 1862 )
            {
            return shiftSatToFriSunToMon( BigDate.toOrdinal( year, 6, 29 ),
                    shift );
            }
        if ( 1863 <= year && year <= 1938 )
            {
            return BigDate.ordinalOfnthXXXDay( 5, BigDate.THURSDAY, year, BigDate.NOV );
            }
        if ( 1939 <= year && year <= 1941 )
            {
            return BigDate.ordinalOfnthXXXDay( 5/* last */, BigDate.THURSDAY, year, BigDate.NOV )
                   - 7/* second last */;
            }
        if ( 1942 <= year )
            {
            return BigDate.ordinalOfnthXXXDay( 4/* fourth */, BigDate.THURSDAY, year, BigDate.NOV );
            }
        return BigDate.NULL_ORDINAL;
        }// end when.
    }
