/*
 * [ChristmasUSA.java]
 *
 * Summary: calculate when Christmas day occurs in the USA.
 *
 * Copyright: (c) 1999-2011 Roedy Green, Canadian Mind Products, http://mindprod.com
 *
 * Licence: This software may be copied and used freely for any purpose but military.
 *          http://mindprod.com/contact/nonmil.html
 *
 * Requires: JDK 1.5+
 *
 * Created with: JetBrains IntelliJ IDEA IDE http://www.jetbrains.com/idea/
 *
 * Version History:
 *  4.2 2008-12-03 add World AIDS day
 */
package com.mindprod.holidays;

import com.mindprod.common11.BigDate;

import static java.lang.System.out;

/**
 * calculate when Christmas day occurs in the USA.
 *
 * @author Roedy Green, Canadian Mind Products
 * @version 4.2 2008-12-03 add World AIDS day
 * @since 1999
 */
public final class ChristmasUSA extends HolInfo
    {
    // -------------------------- PUBLIC INSTANCE  METHODS --------------------------


    /**
     * @inheritDoc
     */
    public String getAuthority()
        {
        return "";
        }

    /**
     * @inheritDoc
     */
    public int getFirstYear( int base )
        {
        return 1;
        }

    /**
     * @inheritDoc
     */
    public String getName()
        {
        return "Christmas in the USA";
        }

    /**
     * @inheritDoc
     */
    public String getRule()
        {
        return "Always on Dec 25.";
        }

    /**
     * Nominally, Christmas is on the 25 in the USA. They are always celebrated on the 25 no matter what day of the week
     * it falls on. However, if it falls on a Saturday or Sunday, you get a day off work on Friday or Monday to
     * compensate.
     *
     * @param year  (-ve means BC, +ve means AD, 0 not permitted.)
     * @param shift true if want date of holiday shifted to nearest weekday.
     * @param base  PROCLAIMED=based on date holiday was officially proclaimed CELEBRATED=based on date holiday was
     *              first celebrated
     *
     * @return ordinal days since 1970-01-01. return NULL_ORDINAL if the holiday was not celebrated in that year.
     */
    public int when( int year, boolean shift, int base )
        {
        if ( !isYearValid( year, base ) )
            {
            return BigDate.NULL_ORDINAL;
            }
        return shiftSatToFriSunToMon( BigDate.toOrdinal( year, 12, 25 ),
                shift );
        }// end when.

    // --------------------------- main() method ---------------------------

    /**
     * Test driver
     *
     * @param args not used
     */
    public static void main( String[] args )
        {
        HolInfo h = new ChristmasUSA();
        out.println( h.getName() );
        out.println( h.getFirstYear( OBSERVED ) );
        out.println( h.getFirstYear( PROCLAIMED ) );
        out.println( h.getRule() );
        out.println( h.getAuthority() );
        BigDate d = new BigDate( h.when( 2007, ACTUAL, OBSERVED ) );
        out.println( d.getYYYY() + "/" + d.getMM() + "/" + d.getDD() );
        d.setOrdinal( h.when( 2007, SHIFTED, OBSERVED ) );
        out.println( d.getYYYY() + "/" + d.getMM() + "/" + d.getDD() );
        }// end main
    }
