/*
 * [DaylightSavingStart.java]
 *
 * Summary: calculate when daylight saving starts in North America.
 *
 * Copyright: (c) 1999-2011 Roedy Green, Canadian Mind Products, http://mindprod.com
 *
 * Licence: This software may be copied and used freely for any purpose but military.
 *          http://mindprod.com/contact/nonmil.html
 *
 * Requires: JDK 1.5+
 *
 * Created with: JetBrains IntelliJ IDEA IDE http://www.jetbrains.com/idea/
 *
 * Version History:
 *  4.2 2008-12-03 add World AIDS day
 */
package com.mindprod.holidays;
// com.mindprod.holidays.DaylightSavingStart

import com.mindprod.common11.BigDate;

import java.util.Date;
import java.util.TimeZone;

/**
 * calculate when daylight saving starts in North America.
 *
 * @author Roedy Green, Canadian Mind Products
 * @version 4.2 2008-12-03 add World AIDS day
 * @since 1999
 */
public final class DaylightSavingStart extends HolInfo
    {
    // -------------------------- PUBLIC INSTANCE  METHODS --------------------------


    /**
     * @inheritDoc
     */
    public String getAuthority()
        {
        return "Sun Java TimeZone tables.";
        }

    /**
     * @inheritDoc
     */
    public int getFirstYear( int base )
        {
        return 1916;
        }

    /**
     * @inheritDoc
     */
    public String getName()
        {
        return "Daylight Saving Time Start";
        }

    /**
     * @inheritDoc
     */
    public String getRule()
        {
        return "DST start date varies with your locale. In Canada and the USA it is usually the second Sunday in March. Set your clock ahead an hour at 2AM.";
        }

    /**
     * @inheritDoc
     */
    public int when( int year, boolean shift, int base )
        {
        if ( !isYearValid( year, base ) )
            {
            return BigDate.NULL_ORDINAL;
            }
        TimeZone tzHere = TimeZone.getDefault();
        if ( !tzHere.useDaylightTime() )
            {
            /* don't use DST ever */
            return BigDate.NULL_ORDINAL;
            }
        final int fromOrd = BigDate.toOrdinal( year, 1, 1 );
        final int toOrd = BigDate.toOrdinal( year, 12, 31 );
        // find first day in year that DST kicked in
        for ( int ord = fromOrd; ord <= toOrd; ord++ )
            {
            BigDate b = new BigDate( ord );
            // midnight
            Date d = b.getDate( tzHere );
            d.setTime( d.getTime() + 12 * 60 * 60 * 1000 );
            if ( tzHere.inDaylightTime( d ) )
                {
                if ( ord == fromOrd )
                    {
                    // we are in the southern hemisphere. Start looking in fall
                    ord = BigDate.toOrdinal( year, 8, 1 );
                    }
                else
                    {
                    return shiftSatToFriSunToMon( ord, shift );
                    }
                }
            }
        return BigDate.NULL_ORDINAL;
        }// end when.
    }
