/*
 * [EarthHour.java]
 *
 * Summary: calculate when Earth Hour occurs.
 *
 * Copyright: (c) 2010-2011 Roedy Green, Canadian Mind Products, http://mindprod.com
 *
 * Licence: This software may be copied and used freely for any purpose but military.
 *          http://mindprod.com/contact/nonmil.html
 *
 * Requires: JDK 1.5+
 *
 * Created with: JetBrains IntelliJ IDEA IDE http://www.jetbrains.com/idea/
 *
 * Version History:
 *  1.0 2010-03-30 initial version
 */
package com.mindprod.holidays;

import com.mindprod.common11.BigDate;

/**
 * calculate when Earth Hour occurs.
 *
 * @author Roedy Green, Canadian Mind Products
 * @version 1.0 2010-03-30 initial version
 * @see EarthDay
 * @since 2010
 */
public final class EarthHour extends HolInfo
    {
    // -------------------------- PUBLIC INSTANCE  METHODS --------------------------


    /**
     * @inheritDoc
     */
    public String getAuthority()
        {
        return "http://www.earthhour.org/FAQ.aspx";
        }

    /**
     * @inheritDoc
     */
    public int getFirstYear( int base )
        {
        return 2007;
        }

    /**
     * @inheritDoc
     */
    public String getName()
        {
        return "Earth Hour";
        }

    /**
     * @inheritDoc
     */
    public String getRule()
        {
        return "Always last Saturday in March, 8:30 9:30 PM local time.";
        }

    /**
     * @inheritDoc
     */
    public int when( int year, boolean shift, int base )
        {
        if ( !isYearValid( year, base ) )
            {
            return BigDate.NULL_ORDINAL;
            }
        return shiftSatToFriSunToMon( BigDate.ordinalOfnthXXXDay( 5 /* last of month */, BigDate.SATURDAY, year, BigDate.MAR ), shift );
        }
    }
