/*
 * [LaborDay.java]
 *
 * Summary: calculate when American Labor day occurs.
 *
 * Copyright: (c) 1999-2011 Roedy Green, Canadian Mind Products, http://mindprod.com
 *
 * Licence: This software may be copied and used freely for any purpose but military.
 *          http://mindprod.com/contact/nonmil.html
 *
 * Requires: JDK 1.5+
 *
 * Created with: JetBrains IntelliJ IDEA IDE http://www.jetbrains.com/idea/
 *
 * Version History:
 *  4.2 2008-12-03 add World AIDS day
 */
package com.mindprod.holidays;

import com.mindprod.common11.BigDate;

/**
 * calculate when American Labor day occurs.
 *
 * @author Roedy Green, Canadian Mind Products
 * @version 4.2 2008-12-03 add World AIDS day
 * @since 1999
 */
public final class LaborDay extends HolInfo
    {
    // -------------------------- PUBLIC INSTANCE  METHODS --------------------------


    /**
     * @inheritDoc
     */
    public String getAuthority()
        {
        return "http://www.dol.gov/opa/aboutdol/laborday.htm";
        }

    /**
     * @inheritDoc
     */
    public int getFirstYear( int base )
        {
        return 1882;
        }

    /**
     * @inheritDoc
     */
    public String getName()
        {
        return "American Labor Day";
        }

    /**
     * @inheritDoc
     */
    public String getRule()
        {
        return "First Monday in September. In 1882..1883 it was celebrated on September 5.";
        }

    /**
     * @inheritDoc
     */
    public int when( int year, boolean shift, int base )
        {
        if ( !isYearValid( year, base ) )
            {
            return BigDate.NULL_ORDINAL;
            }
        if ( year < 1882 )
            {
            return BigDate.NULL_ORDINAL;
            }
        if ( 1882 <= year && year <= 1883 )
            {
            return shiftSatToFriSunToMon( BigDate.toOrdinal( year, 9, 5 ),
                    shift );
            }
        if ( 1884 <= year )
            {
            return BigDate.ordinalOfnthXXXDay( 1, BigDate.MONDAY, year, BigDate.SEP );
            }
        return BigDate.NULL_ORDINAL;
        }// end when.
    }
