/*
 * [VeteransDay.java]
 *
 * Summary: calculate when Veteran's Day occurs.
 *
 * Copyright: (c) 1999-2011 Roedy Green, Canadian Mind Products, http://mindprod.com
 *
 * Licence: This software may be copied and used freely for any purpose but military.
 *          http://mindprod.com/contact/nonmil.html
 *
 * Requires: JDK 1.5+
 *
 * Created with: JetBrains IntelliJ IDEA IDE http://www.jetbrains.com/idea/
 *
 * Version History:
 *  4.2 2008-12-03 add World AIDS day
 */
package com.mindprod.holidays;

import com.mindprod.common11.BigDate;

/**
 * calculate when Veteran's Day occurs.
 *
 * @author Roedy Green, Canadian Mind Products
 * @version 4.2 2008-12-03 add World AIDS day
 * @since 1999
 */
public final class VeteransDay extends HolInfo
    {
    // -------------------------- PUBLIC INSTANCE  METHODS --------------------------


    /**
     * @inheritDoc
     */
    public String getAuthority()
        {
        return "http://www.usis.usemb.se/Holidays/celebrate/Veterans.htm";
        }

    /**
     * @inheritDoc
     */
    public int getFirstYear( int base )
        {
        switch ( base )
            {
            default:
            case OBSERVED:
                return 1918;// end of WWI
            case PROCLAIMED:
                return 1938;
            }
        }

    /**
     * @inheritDoc
     */
    public String getName()
        {
        return "Veterans Day";
        }

    /**
     * @inheritDoc
     */
    public String getRule()
        {
        return "Always November 11";
        }

    /**
     * @inheritDoc
     */
    public int when( int year, boolean shift, int base )
        {
        if ( !isYearValid( year, base ) )
            {
            return BigDate.NULL_ORDINAL;
            }
        return shiftSatToFriSunToMon( BigDate.toOrdinal( year, 11, 11 ),
                shift );
        }// end when.
    }
